# Used for QuickPrint (qp).  Change the uncommented printf statement in the first block to get a different font.

BEGIN {
	printf("E&k3G(s0p12v0s0b4099T&a06L") # courier 12 point LaserJets
	#printf("E&k3G(s1p12v0s0b16901T&a06L") # times roman med
	#printf("E&k3G(s0p10h12v0s0b3T&a06L") # courier 12 point DeskJets
	#printf("E&k3G(s1p14v0s4b4362T&a06L") # albertus extra bold 
	}
{
print $0
}

END {
	printf("E")
}
